@@ -30,7 +30,6 @@
 #include "ephy-string.h"
 #include "ephy-view-source-handler.h"
 
-#include <evince-document.h>
 #include <glib/gi18n.h>
 #include <jsc/jsc.h>
 #include <libsoup/soup.h>
@@ -416,27 +415,3 @@ ephy_embed_utils_shutdown (void)
   g_clear_pointer (&non_search_regex, g_regex_unref);
   g_clear_pointer (&domain_regex, g_regex_unref);
 }
-
-gboolean
-ephy_embed_utils_mime_type_is_supported_evince_document (const char *mime_type)
-{
-  GList *doc_types = ev_backends_manager_get_all_types_info ();
-  GList *l;
-  gboolean found = FALSE;
-
-  for (l = doc_types; l != NULL && !found; l = l->next) {
-    EvTypeInfo *info = (EvTypeInfo *)l->data;
-    guint i;
-
-    for (i = 0; info->mime_types[i] != NULL; ++i) {
-      if (g_ascii_strcasecmp (mime_type, info->mime_types[i]) == 0) {
-        found = TRUE;
-        break;
-      }
-    }
-  }
-
-  g_list_free (doc_types);
-
-  return found;
-}
