@@ -55,7 +55,6 @@ struct _EphyWebExtension {
   EphyUriTester *uri_tester;
 
   WebKitScriptWorld *script_world;
-  WebKitWebPage *web_page;
 
   gboolean is_private_profile;
 };
@@ -87,11 +86,13 @@ static const char introspection_xml[] =
   "  <method name='PasswordQueryResponse'>"
   "    <arg type='s' name='username' direction='in'/>"
   "    <arg type='s' name='password' direction='in'/>"
-  "    <arg type='i' name='id' direction='in'/>"
+  "    <arg type='i' name='promise_id' direction='in'/>"
+  "    <arg type='t' name='page_id' direction='in'/>"
   "  </method>"
   "  <method name='PasswordQueryUsernamesResponse'>"
   "    <arg type='as' name='users' direction='in'/>"
-  "    <arg type='i' name='id' direction='in'/>"
+  "    <arg type='i' name='promise_id' direction='in'/>"
+  "    <arg type='t' name='page_id' direction='in'/>"
   "  </method>"
   " </interface>"
   "</node>";
@@ -103,7 +104,7 @@ should_use_adblocker (const char *request_uri,
                       const char *page_uri,
                       const char *redirected_request_uri)
 {
-  if (!g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_ADBLOCK))
+  if (!g_settings_get_boolean (EPHY_SETTINGS_WEB_EXTENSION_WEB, EPHY_PREFS_WEB_ENABLE_ADBLOCK))
     return FALSE;
 
   /* Always load the main resource... */
@@ -150,7 +151,7 @@ web_page_send_request (WebKitWebPage     *web_page,
   page_uri = webkit_web_page_get_uri (web_page);
   redirected_response_uri = redirected_response ? webkit_uri_response_get_uri (redirected_response) : NULL;
 
-  if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DO_NOT_TRACK)) {
+  if (g_settings_get_boolean (EPHY_SETTINGS_WEB_EXTENSION_WEB, EPHY_PREFS_WEB_DO_NOT_TRACK)) {
     SoupMessageHeaders *headers = webkit_uri_request_get_http_headers (request);
     if (headers) {
       /* Do Not Track header. '1' means 'opt-out'. See:
@@ -275,7 +276,7 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
                                           G_TYPE_STRING, 2,
                                           G_TYPE_UINT64, G_TYPE_BOOLEAN);
   remember_passwords = !extension->is_private_profile &&
-                       g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_REMEMBER_PASSWORDS);
+                       g_settings_get_boolean (EPHY_SETTINGS_WEB_EXTENSION_WEB, EPHY_PREFS_WEB_REMEMBER_PASSWORDS);
   js_result = jsc_value_object_invoke_method (js_ephy,
                                               "formControlsAssociated",
                                               G_TYPE_UINT64, webkit_web_page_get_id (web_page),
@@ -388,7 +389,6 @@ ephy_web_extension_page_created_cb (EphyWebExtension *extension,
   g_object_unref (js_context);
 
   page_id = webkit_web_page_get_id (web_page);
-  extension->web_page = web_page;
   if (extension->dbus_connection)
     ephy_web_extension_emit_page_created (extension, page_id);
   else
@@ -409,11 +409,12 @@ ephy_web_extension_page_created_cb (EphyWebExtension *extension,
 }
 
 static JSCValue *
-get_password_manager (EphyWebExtension *self)
+get_password_manager (EphyWebExtension *self, guint64 page_id)
 {
-  g_assert (self->web_page);
-
-  WebKitFrame *frame = webkit_web_page_get_main_frame (self->web_page);
+  WebKitWebPage *page = webkit_web_extension_get_page (self->extension, page_id);
+  if (page == NULL)
+    return NULL;
+  WebKitFrame *frame = webkit_web_page_get_main_frame (page);
   JSCContext *context = webkit_frame_get_js_context_for_script_world (frame,
                           self->script_world);
   g_autoptr(JSCValue) ephy = jsc_context_get_value (context, "Ephy");
@@ -493,27 +494,33 @@ handle_method_call (GDBusConnection       *connection,
     g_dbus_method_invocation_return_value (invocation, NULL);
   } else if (g_strcmp0 (method_name, "PasswordQueryUsernamesResponse") == 0) {
     g_autofree const char **users;
-    g_autoptr(JSCValue) ret;
-    gint32 id;
+    g_autoptr(JSCValue) ret = NULL;
+    gint32 promise_id;
+    guint64 page_id;
 
     users = g_variant_get_strv (g_variant_get_child_value (parameters, 0), NULL);
-    g_variant_get_child (parameters, 1, "i", &id);
+    g_variant_get_child (parameters, 1, "i", &promise_id);
+    g_variant_get_child (parameters, 2, "t", &page_id);
 
-    g_autoptr(JSCValue) password_manager = get_password_manager (extension);
-    ret = jsc_value_object_invoke_method (password_manager, "_onQueryUsernamesResponse",
-                                          G_TYPE_STRV, users, G_TYPE_INT, id, G_TYPE_NONE);
+    g_autoptr(JSCValue) password_manager = get_password_manager (extension, page_id);
+    if (password_manager != NULL)
+      ret = jsc_value_object_invoke_method (password_manager, "_onQueryUsernamesResponse",
+                                            G_TYPE_STRV, users, G_TYPE_INT, promise_id, G_TYPE_NONE);
   } else if (g_strcmp0 (method_name, "PasswordQueryResponse") == 0) {
     const char *username;
     const char *password;
-    gint32 id;
-    g_autoptr(JSCValue) ret;
+    gint32 promise_id;
+    guint64 page_id;
+    g_autoptr(JSCValue) ret = NULL;
 
-    g_variant_get (parameters, "(&s&si)", &username, &password, &id);
-    g_autoptr(JSCValue) password_manager = get_password_manager (extension);
-    ret = jsc_value_object_invoke_method (password_manager, "_onQueryResponse",
-                                          G_TYPE_STRING, username,
-                                          G_TYPE_STRING, password,
-                                          G_TYPE_INT, id, G_TYPE_NONE);
+
+    g_variant_get (parameters, "(&s&sit)", &username, &password, &promise_id, &page_id);
+    g_autoptr(JSCValue) password_manager = get_password_manager (extension, page_id);
+    if (password_manager != NULL)
+      ret = jsc_value_object_invoke_method (password_manager, "_onQueryResponse",
+                                            G_TYPE_STRING, username,
+                                            G_TYPE_STRING, password,
+                                            G_TYPE_INT, promise_id, G_TYPE_NONE);
   }
 }
 
@@ -763,7 +770,7 @@ window_object_cleared_cb (WebKitScriptWorld *world,
 
   js_function = jsc_value_new_function (js_context,
                                         "isWebApplication",
-                                        G_CALLBACK (ephy_dot_dir_is_web_application), NULL, NULL,
+                                        G_CALLBACK (ephy_profile_dir_is_web_application), NULL, NULL,
                                         G_TYPE_BOOLEAN, 0,
                                         G_TYPE_NONE);
   jsc_value_object_set_property (js_ephy, "isWebApplication", js_function);
