@@ -390,14 +390,14 @@ window_cmd_import_bookmarks (GSimpleAction *action,
   GtkTreeModel *tree_model;
   GtkCellRenderer *cell_renderer;
 
-  dialog = gtk_dialog_new_with_buttons (_("Import Bookmarks"),
-                                        GTK_WINDOW (window),
-                                        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_USE_HEADER_BAR,
-                                        _("_Cancel"),
-                                        GTK_RESPONSE_CANCEL,
-                                        _("Ch_oose File"),
-                                        GTK_RESPONSE_OK,
-                                        NULL);
+  dialog = hdy_dialog_new (GTK_WINDOW (window));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Import Bookmarks"));
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                          _("_Cancel"),
+                          GTK_RESPONSE_CANCEL,
+                          _("Ch_oose File"),
+                          GTK_RESPONSE_OK,
+                          NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
@@ -668,9 +668,7 @@ window_cmd_navigation (GSimpleAction *action,
   web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
 
   if (strstr (g_action_get_name (G_ACTION (action)), "back")) {
-    ephy_embed_set_mode (embed, EPHY_EMBED_MODE_WEB_VIEW);
     webkit_web_view_go_back (web_view);
-
     gtk_widget_grab_focus (GTK_WIDGET (embed));
   } else {
     webkit_web_view_go_forward (web_view);
@@ -748,29 +746,6 @@ window_cmd_stop (GSimpleAction *action,
   webkit_web_view_stop_loading (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 }
 
-static gboolean
-event_with_shift (void)
-{
-  GdkEvent *event;
-  GdkEventType type = 0;
-  guint state = 0;
-
-  event = gtk_get_current_event ();
-  if (event) {
-    type = event->type;
-
-    if (type == GDK_BUTTON_RELEASE) {
-      state = event->button.state;
-    } else if (type == GDK_KEY_PRESS || type == GDK_KEY_RELEASE) {
-      state = event->key.state;
-    }
-
-    gdk_event_free (event);
-  }
-
-  return (state & GDK_SHIFT_MASK) != 0;
-}
-
 void
 window_cmd_reload (GSimpleAction *action,
                    GVariant      *parameter,
@@ -780,17 +755,31 @@ window_cmd_reload (GSimpleAction *action,
   EphyEmbed *embed;
   WebKitWebView *view;
 
-  embed = ephy_embed_container_get_active_child
-            (EPHY_EMBED_CONTAINER (window));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
 
   gtk_widget_grab_focus (GTK_WIDGET (embed));
 
   view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
-  if (event_with_shift ())
-    webkit_web_view_reload_bypass_cache (view);
-  else
-    webkit_web_view_reload (view);
+  webkit_web_view_reload (view);
+}
+
+void
+window_cmd_reload_bypass_cache (GSimpleAction *action,
+                                GVariant      *parameter,
+                                gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  WebKitWebView *view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+
+  view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+  webkit_web_view_reload_bypass_cache (view);
 }
 
 void window_cmd_combined_stop_reload (GSimpleAction *action,
@@ -1177,14 +1166,15 @@ set_default_application_title (EphyApplicationDialogData *data,
     uri = soup_uri_new (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)));
     host = soup_uri_get_host (uri);
 
-    if (host != NULL && host[0] != '\0')
+    if (host != NULL && host[0] != '\0') {
       title = get_special_case_application_title_for_host (host);
 
-    if (title == NULL || title[0] == '\0') {
-      if (g_str_has_prefix (host, "www."))
-        title = g_strdup (host + strlen ("www."));
-      else
-        title = g_strdup (host);
+      if (title == NULL || title[0] == '\0') {
+        if (g_str_has_prefix (host, "www."))
+          title = g_strdup (host + strlen ("www."));
+        else
+          title = g_strdup (host);
+      }
     }
 
     soup_uri_free (uri);
@@ -1221,10 +1211,14 @@ notify_launch_cb (NotifyNotification *notification,
                   char               *action,
                   gpointer            user_data)
 {
-  char *desktop_file = user_data;
+  g_autofree char *desktop_file = user_data;
 
-  ephy_file_launch_desktop_file (desktop_file, NULL, 0, NULL);
-  g_free (desktop_file);
+  /* We can't get here under flatpak because all web app functionality
+   * is disabled when running under flatpak.
+   */
+  ephy_file_launch_desktop_file (desktop_file,
+                                 gtk_get_current_event_time (),
+                                 EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
 }
 
 static gboolean
@@ -1256,6 +1250,42 @@ confirm_web_application_overwrite (GtkWindow *parent, const char *title)
   return response == GTK_RESPONSE_OK;
 }
 
+static void
+session_bus_ready_cb (GObject      *source,
+                      GAsyncResult *res,
+                      gpointer      user_data)
+{
+  g_autoptr(GDBusConnection) connection = g_bus_get_finish (res, NULL);
+  g_autofree gchar *desktop_file = user_data;
+  g_autofree gchar *app_id = NULL;
+  GVariant *app;
+
+  if (!connection)
+    return;
+
+  app_id = g_path_get_basename (desktop_file);
+  app = g_variant_new_string (app_id);
+
+  g_dbus_connection_call (connection,
+                          "org.gnome.Shell",
+                          "/org/gnome/Shell",
+                          "org.gnome.Shell",
+                          "FocusApp",
+                          g_variant_new_tuple (&app, 1),
+                          NULL,
+                          G_DBUS_CALL_FLAGS_NO_AUTO_START,
+                          -1,
+                          NULL,
+                          NULL,
+                          NULL);
+}
+
+static void
+ephy_focus_desktop_app (const char *desktop_file)
+{
+  g_bus_get (G_BUS_TYPE_SESSION, NULL, session_bus_ready_cb, g_strdup (desktop_file));
+}
+
 static void
 dialog_save_as_application_response_cb (GtkDialog                 *dialog,
                                         gint                       response,
@@ -1263,8 +1293,8 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
 {
   if (response == GTK_RESPONSE_OK) {
     const char *app_name;
-    char *app_id;
-    char *desktop_file;
+    g_autofree gchar *app_id = NULL;
+    g_autofree gchar *desktop_file = NULL;
     char *message;
     NotifyNotification *notification;
 
@@ -1274,10 +1304,8 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
     if (ephy_web_application_exists (app_id)) {
       if (confirm_web_application_overwrite (GTK_WINDOW (dialog), app_name))
         ephy_web_application_delete (app_id);
-      else {
-        g_free (app_id);
+      else
         return;
-      }
     }
 
     /* Create Web Application, including a new profile and .desktop file. */
@@ -1285,7 +1313,6 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
                                                 webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)),
                                                 app_name,
                                                 gtk_image_get_pixbuf (GTK_IMAGE (data->image)));
-    g_free (app_id);
 
     if (desktop_file)
       message = g_strdup_printf (_("The application “%s” is ready to be used"),
@@ -1304,7 +1331,8 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
                                       g_path_get_basename (desktop_file),
                                       NULL);
       notify_notification_set_icon_from_pixbuf (notification, gtk_image_get_pixbuf (GTK_IMAGE (data->image)));
-      g_free (desktop_file);
+
+      ephy_focus_desktop_app (desktop_file);
     }
 
     notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
@@ -1872,7 +1900,7 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
     goto out;
   }
 
-  if (!ephy_file_launch_handler ("text/plain", file, gtk_get_current_event_time ())) {
+  if (!ephy_file_launch_handler (file, gtk_get_current_event_time ())) {
     /* Fallback to view the source inside the browser */
     EphyEmbed *embed;
 
@@ -2059,7 +2087,7 @@ window_cmd_page_source (GSimpleAction *action,
     GFile *file;
 
     file = g_file_new_for_uri (address);
-    ephy_file_launch_handler ("text/plain", file, user_time);
+    ephy_file_launch_handler (file, user_time);
 
     g_object_unref (file);
   } else {
@@ -2597,3 +2625,42 @@ window_cmd_homepage_new_tab (GSimpleAction *action,
 
   gtk_widget_grab_focus (GTK_WIDGET (embed));
 }
+
+static void
+clipboard_text_received_cb (GtkClipboard *clipboard,
+                            const gchar  *text,
+                            EphyWindow   *window)
+{
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                              EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
+                              NULL,
+                              0);
+
+  web_view = ephy_embed_get_web_view (embed);
+  ephy_web_view_load_url (web_view, text);
+
+  ephy_embed_container_set_active_child (EPHY_EMBED_CONTAINER (window), embed);
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+
+  g_object_unref (window);
+}
+
+void
+window_cmd_new_tab_from_clipboard (GSimpleAction *action,
+                                   GVariant      *parameter,
+                                   gpointer       user_data)
+{
+  EphyWindow *ephy_window = EPHY_WINDOW (user_data);
+  GtkClipboard *clipboard;
+
+  clipboard = gtk_clipboard_get_default (gdk_display_get_default ());
+  gtk_clipboard_request_text (clipboard,
+                              (GtkClipboardTextReceivedFunc)clipboard_text_received_cb,
+                              g_object_ref (ephy_window));
+}
